diff mbox series

firmware: ti_sci: Use system_state to determine polling

Message ID 20230619131236.3285069-1-nm@ti.com (mailing list archive)
State New, archived
Headers show
Series firmware: ti_sci: Use system_state to determine polling | expand

Commit Message

Nishanth Menon June 19, 2023, 1:12 p.m. UTC
Commit b9e8a7d950ff ("firmware: ti_sci: Switch transport to polled
mode during system suspend") aims to resolve issues with tisci
operations during system suspend operation. However, the system may
enter a no_irq stage in various other usage modes, including power-off
and restart. To determine if polling mode is appropriate, use the
system_state instead.

While at this, drop the unused is_suspending state variable and
related helpers.

Reported-by: Francesco Dolcini <francesco@dolcini.it>
Reported-by: Wadim Egorov <w.egorov@phytec.de>
Link: https://lore.kernel.org/all/ZGeHMjlnob2GFyHF@francesco-nb.int.toradex.com/
Signed-off-by: Nishanth Menon <nm@ti.com>
---
 drivers/firmware/ti_sci.c | 35 ++---------------------------------
 1 file changed, 2 insertions(+), 33 deletions(-)

Comments

Francesco Dolcini June 19, 2023, 9:20 p.m. UTC | #1
On Mon, Jun 19, 2023 at 08:12:36AM -0500, Nishanth Menon wrote:
> Commit b9e8a7d950ff ("firmware: ti_sci: Switch transport to polled
> mode during system suspend") aims to resolve issues with tisci
> operations during system suspend operation. However, the system may
> enter a no_irq stage in various other usage modes, including power-off
> and restart. To determine if polling mode is appropriate, use the
> system_state instead.
> 
> While at this, drop the unused is_suspending state variable and
> related helpers.
> 
> Reported-by: Francesco Dolcini <francesco@dolcini.it>
> Reported-by: Wadim Egorov <w.egorov@phytec.de>
> Link: https://lore.kernel.org/all/ZGeHMjlnob2GFyHF@francesco-nb.int.toradex.com/
> Signed-off-by: Nishanth Menon <nm@ti.com>

s/Link/Closes/

Fixes: b9e8a7d950ff ("firmware: ti_sci: Switch transport to polled mode during system suspend")

>  drivers/firmware/ti_sci.c | 35 ++---------------------------------
>  1 file changed, 2 insertions(+), 33 deletions(-)
> 
> diff --git a/drivers/firmware/ti_sci.c b/drivers/firmware/ti_sci.c
> index 039d92a595ec..dc7906e38f09 100644
> --- a/drivers/firmware/ti_sci.c
> +++ b/drivers/firmware/ti_sci.c
...
> -static DEFINE_SIMPLE_DEV_PM_OPS(ti_sci_pm_ops, ti_sci_suspend, ti_sci_resume);

You need to stop using ti_sci_pm_ops in ti_sci_driver, without this
change it does not build.

diff --git a/drivers/firmware/ti_sci.c b/drivers/firmware/ti_sci.c
index dc7906e38f09..91aaa0ca9bde 100644
--- a/drivers/firmware/ti_sci.c
+++ b/drivers/firmware/ti_sci.c
@@ -3485,7 +3485,6 @@ static struct platform_driver ti_sci_driver = {
        .driver = {
                   .name = "ti-sci",
                   .of_match_table = of_match_ptr(ti_sci_of_match),
-                  .pm = &ti_sci_pm_ops,
        },
 };


With that change I was able to do a quick test on a patched v6.4-rc7 and
it seems to solve the issue.

Tested-by: Francesco Dolcini <francesco.dolcini@toradex.com> # Toradex Verdin AM62

Francesco
diff mbox series

Patch

diff --git a/drivers/firmware/ti_sci.c b/drivers/firmware/ti_sci.c
index 039d92a595ec..dc7906e38f09 100644
--- a/drivers/firmware/ti_sci.c
+++ b/drivers/firmware/ti_sci.c
@@ -97,7 +97,6 @@  struct ti_sci_desc {
  * @node:	list head
  * @host_id:	Host ID
  * @users:	Number of users of this instance
- * @is_suspending: Flag set to indicate in suspend path.
  */
 struct ti_sci_info {
 	struct device *dev;
@@ -116,7 +115,6 @@  struct ti_sci_info {
 	u8 host_id;
 	/* protected by ti_sci_list_mutex */
 	int users;
-	bool is_suspending;
 };
 
 #define cl_to_ti_sci_info(c)	container_of(c, struct ti_sci_info, cl)
@@ -418,14 +416,14 @@  static inline int ti_sci_do_xfer(struct ti_sci_info *info,
 
 	ret = 0;
 
-	if (!info->is_suspending) {
+	if (system_state <= SYSTEM_RUNNING) {
 		/* And we wait for the response. */
 		timeout = msecs_to_jiffies(info->desc->max_rx_timeout_ms);
 		if (!wait_for_completion_timeout(&xfer->done, timeout))
 			ret = -ETIMEDOUT;
 	} else {
 		/*
 		 * If we are suspending, we cannot use wait_for_completion_timeout
 		 * during noirq phase, so we must manually poll the completion.
 		 */
 		ret = read_poll_timeout_atomic(try_wait_for_completion, done_state,
@@ -3281,35 +3279,6 @@  static int tisci_reboot_handler(struct notifier_block *nb, unsigned long mode,
 	return NOTIFY_BAD;
 }
 
-static void ti_sci_set_is_suspending(struct ti_sci_info *info, bool is_suspending)
-{
-	info->is_suspending = is_suspending;
-}
-
-static int ti_sci_suspend(struct device *dev)
-{
-	struct ti_sci_info *info = dev_get_drvdata(dev);
-	/*
-	 * We must switch operation to polled mode now as drivers and the genpd
-	 * layer may make late TI SCI calls to change clock and device states
-	 * from the noirq phase of suspend.
-	 */
-	ti_sci_set_is_suspending(info, true);
-
-	return 0;
-}
-
-static int ti_sci_resume(struct device *dev)
-{
-	struct ti_sci_info *info = dev_get_drvdata(dev);
-
-	ti_sci_set_is_suspending(info, false);
-
-	return 0;
-}
-
-static DEFINE_SIMPLE_DEV_PM_OPS(ti_sci_pm_ops, ti_sci_suspend, ti_sci_resume);
-
 /* Description for K2G */
 static const struct ti_sci_desc ti_sci_pmmc_k2g_desc = {
 	.default_host_id = 2,
@@ -418,14 +416,14 @@  static inline int ti_sci_do_xfer(struct ti_sci_info *info,
 	} else {
 		/*
-		 * If we are suspending, we cannot use wait_for_completion_timeout
+		 * If we are !running, we cannot use wait_for_completion_timeout
 		 * during noirq phase, so we must manually poll the completion.
 		 */